{ **************************************************************************** }
{ *                                                                          * }
{ * PENGUIN LAND (Bandai, LSI Game Double Play) Simulator                    * }
{ * by Luca "MADrigal" Antignano, April 2000 (revision: August 2021)         * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit manual;
{ this unit contains the instructions manual browser implementation            }

interface

uses
  Windows, Forms, Graphics, Controls, Classes, ExtCtrls;

type
  TManualForm = class(TForm)
    // manual pages
    im_page: TImage;

    // controls
    btn_up_down:    TImage;         btn_up_up:     TImage;
    btn_down_up:    TImage;         btn_down_down: TImage;

    btn_close_down: TImage;         btn_close_up:  TImage;
    sh_page_top: TShape;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate            (Sender: TObject);
    procedure FormShow              (Sender: TObject);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Show_Buttons;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure sh_page_topMouseDown  (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure sh_page_topMouseMove  (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_up_upMouseDown    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_up_upMouseUp      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_down_upMouseDown  (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_down_upMouseUp    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_close_upMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_close_upMouseUp   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown           (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp             (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_up_down;          procedure Inp_up_up;
    procedure Inp_down_down;        procedure Inp_down_up;
    procedure Inp_esc_down;         procedure Inp_esc_up;
  end;

const
  MANUALPAGES = 1;

var
{ * main objects * }
  ManualForm:       TManualForm;                                                // main form
  iMouseX, iMouseY: integer;                                                    // cursor coordinates (to drag main form)

implementation

uses
{}unit1, gfxinit;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TManualForm.FormCreate(Sender: TObject);
{ no array of images in this manual                                            }
begin
  // coordinates setting
  // im_page.Top  := 0; {set in Gfx_Manual_Init}
  // im_page.Left := - im_page.Width + ClientWidth; {set in Gfx_Manual_Init}

  // loads graphics resources and initializes the instructions manual graphics
  Gfx_Manual_Init;

{}// visualization
{}Position := poScreenCenter;
{}FormStyle := fsStayOnTop;
{}
{}// variables init
{}DoubleBuffered := true;                                                       // enables double buffering for rendering of this form
end;





procedure TManualForm.FormShow(Sender: TObject);
{ displays buttons                                                             }
begin
  Vid_Show_Buttons;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }
procedure TManualForm.Vid_Show_Buttons;
{ enables/disables arrow buttons, depending on the currently viewed area of    }
{ the manual                                                                   }
begin
  btn_up_up.Visible := im_page.Top < 0;
  btn_up_down.Visible := btn_up_up.Visible;

  btn_down_up.Visible := im_page.Top <> ClientHeight - im_page.Height;
  btn_down_down.Visible := btn_down_up.Visible;
end;




{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TManualForm.sh_page_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
begin
  if Button = mbLeft then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TManualForm.sh_page_topMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
begin
  if ssLeft in Shift then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TManualForm.btn_up_upMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ up button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_up_down;
end;





procedure TManualForm.btn_up_upMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ up button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_up_up;
end;





procedure TManualForm.btn_down_upMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ down button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_down_down;
end;





procedure TManualForm.btn_down_upMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ down button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_down_up;
end;





procedure TManualForm.btn_close_upMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ close button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_esc_down;
end;





procedure TManualForm.btn_close_upMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ close button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_esc_up;
end;





procedure TManualForm.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  case Key of
    VK_UP:     Inp_up_down;
    VK_DOWN:   Inp_down_down;
    VK_ESCAPE: Inp_esc_down;
  end; // case..of
end;





procedure TManualForm.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  case Key of
    VK_UP:     Inp_up_up;
    VK_DOWN:   Inp_down_up;
    VK_ESCAPE: Inp_esc_up;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TManualForm.Inp_up_down;
{ up button: PRESS                                                             }
  var
    i: integer;
begin
  btn_up_up.visible := false;

  i := im_page.Top;

  if (i >= -(im_page.Height - ClientHeight))
  and (i < 0) then
    begin
      Inc(i, 100);

      if i > 0 then
        i := 0;
    end;

  im_page.Top := i;
end;





procedure TManualForm.Inp_up_up;
{ up button: RELEASE                                                           }
begin
  btn_up_up.visible := true;

  Vid_Show_Buttons;
end;





procedure TManualForm.Inp_down_down;
{ down button: PRESS                                                           }
  var
    i: integer;
begin
  btn_down_up.visible := false;

  i := im_page.Top;

  if (i > -(im_page.Height - ClientHeight))
  and (i <= 0) then
    begin
      Dec(i, 100);

      if i < -(im_page.Height - ClientHeight) then
         i := - im_page.Height + ClientHeight;
    end;

  im_page.Top := i;
end;





procedure TManualForm.Inp_down_up;
{ down button: RELEASE                                                         }
begin
  btn_down_up.visible := true;

  Vid_Show_Buttons;
end;





procedure TManualForm.Inp_esc_down;
{ close button: PRESS                                                          }
begin
  btn_close_up.visible := false;
end;





procedure TManualForm.Inp_esc_up;
{ close button: RELEASE                                                        }
begin
  btn_close_up.visible := true;

  Close;
end;





end.

