{ **************************************************************************** }
{ *                                                                          * }
{ * PENGUIN LAND (Bandai, LSI Game Double Play) Simulator                    * }
{ * by Luca "MADrigal" Antignano, April 2000 (revision: August 2021)         * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:             TImage;
    btn_game_menu_background:      TImage;

    im_backdrop:                   TImage;
    im_backdrop_logo:              TImage;

    // electronic game
    im_background:                 TImage;
    im_left_background_disabled:   TImage; im_right_background_disabled:   TImage;
    im_left_background_screen_top: TImage; im_right_background_screen_top: TImage;

    btn_power_down_on:             TImage;  btn_power_top:                 TImage;
    btn_select_down_1P:            TImage;  btn_select_top:                TImage;
    btn_sound_down_on:             TImage;  btn_sound_top:                 TImage;

    btn_penguin_atk_down:          TImage;  btn_penguin_atk_top:           TImage;
    btn_penguin_left_down:         TImage;  btn_penguin_left_top:          TImage;
    btn_penguin_right_down:        TImage;  btn_penguin_right_top:         TImage;

    btn_seal_atk_down:             TImage;  btn_seal_atk_top:              TImage;
    btn_seal_left_down:            TImage;  btn_seal_left_top:             TImage;
    btn_seal_right_down:           TImage;  btn_seal_right_top:            TImage;

{ * LCD SCREEN GRAPHICS (SPRITES) * LEFT DISPLAY (PENGUIN) *                 * }

    // on-screen panel (score for seal and penguin)
    im_left_panel_score_seal_1: TImage; im_left_panel_score_seal_2: TImage;
    im_left_panel_score_peng_1: TImage; im_left_panel_score_peng_2: TImage;

    // on-screen items
    im_left_vulture_1_5: TImage; im_left_vulture_2_5: TImage; im_left_vulture_3_5: TImage; im_left_vulture_4_5: TImage;

    im_left_seal_wall_2: TImage; im_left_seal_wall_3: TImage; im_left_seal_wall_4: TImage;

    im_left_peng_1: TImage; im_left_peng_2_body: TImage; im_left_peng_3_body: TImage; im_left_peng_4_body: TImage; im_left_peng_5: TImage; im_left_peng_6_climb: TImage; im_left_peng_7_eat: TImage;
                            im_left_peng_2_down: TImage; im_left_peng_3_down: TImage; im_left_peng_4_down: TImage;                                                       im_left_peng_7_eat_hand: TImage;
                            im_left_peng_2_up: TImage;   im_left_peng_3_up: TImage;   im_left_peng_4_up: TImage;

    im_left_peng_blown: TImage; im_left_seal_blown: TImage;

    im_left_seal_eat: TImage; im_left_seal_eat_hand: TImage;

    im_left_bomb_1_5: TImage; im_left_bomb_2_5: TImage; im_left_bomb_3_5: TImage; im_left_bomb_4_5: TImage;
                              im_left_bomb_2_4: TImage; im_left_bomb_3_4: TImage; im_left_bomb_4_4: TImage;
                              im_left_bomb_2_3: TImage; im_left_bomb_3_3: TImage; im_left_bomb_4_3: TImage;
                              im_left_bomb_2_2: TImage; im_left_bomb_3_2: TImage; im_left_bomb_4_2: TImage;
                              im_left_bomb_2_1: TImage; im_left_bomb_3_1: TImage; im_left_bomb_4_1: TImage;

    im_left_explosion: TImage;

    im_left_block_0: TImage; im_left_block_1: TImage; im_left_block_2: TImage; im_left_block_3: TImage; im_left_block_4: TImage; im_left_block_5_down: TImage;
                                                                                                                                 im_left_block_5_middle: TImage;
                                                                                                                                 im_left_block_5_top: TImage;

{ * LCD SCREEN GRAPHICS (SPRITES) * RIGHT DISPLAY (SEAL) *                   * }

    // on-screen panel (score for seal and penguin)
    im_right_panel_score_seal_1: TImage; im_right_panel_score_seal_2: TImage;
    im_right_panel_score_peng_1: TImage; im_right_panel_score_peng_2: TImage;

    // on-screen items
    im_right_vulture_5_6: TImage; im_right_vulture_4_6: TImage; im_right_vulture_3_6: TImage; im_right_vulture_2_6: TImage;

    im_right_peng_wall_4: TImage; im_right_peng_wall_3: TImage; im_right_peng_wall_2: TImage;

    im_right_seal_7_eat: TImage;      im_right_seal_6_climb: TImage; im_right_seal_5: TImage; im_right_seal_4_body: TImage; im_right_seal_3_body: TImage; im_right_seal_2_body: TImage; im_right_seal_1: TImage;
    im_right_seal_7_eat_hand: TImage;                                                         im_right_seal_4_down: TImage; im_right_seal_3_down: TImage; im_right_seal_2_down: TImage;
                                                                                              im_right_seal_4_up: TImage;   im_right_seal_3_up: TImage;   im_right_seal_2_up: TImage;

    im_right_seal_blown: TImage; im_right_peng_blown: TImage;

    im_right_peng_eat: TImage; im_right_peng_eat_hand: TImage;

    im_right_bomb_5_6: TImage; im_right_bomb_4_6: TImage;  im_right_bomb_3_6: TImage;  im_right_bomb_2_6: TImage;
                               im_right_bomb_4_7: TImage;  im_right_bomb_3_7: TImage;  im_right_bomb_2_7: TImage;
                               im_right_bomb_4_8: TImage;  im_right_bomb_3_8: TImage;  im_right_bomb_2_8: TImage;
                               im_right_bomb_4_9: TImage;  im_right_bomb_3_9: TImage;  im_right_bomb_2_9: TImage;
                               im_right_bomb_4_10: TImage; im_right_bomb_3_10: TImage; im_right_bomb_2_10: TImage;

    im_right_explosion: TImage;

    im_right_block_5_down: TImage;   im_right_block_4: TImage; im_right_block_3: TImage; im_right_block_2: TImage; im_right_block_1: TImage; im_right_block_0: TImage;
    im_right_block_5_middle: TImage;
    im_right_block_5_top: TImage;

{ * LCD SCREEN GRAPHICS (SPRITES) * COMMON (DIGITS) *                        * }

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage;

{ * TIMERS                                                                   * }
    Timer_Block_Generate:   TTimer;
    Timer_Blocks_Bonus:     TTimer;
    Timer_CPU_AI_Execute:   TTimer;
    Timer_Game:             TTimer;
    Timer_Game_Miss:        TTimer;
    Timer_Game_Over:        TTimer;
    Timer_Game_Start:       TTimer;
    Timer_Move_Arm_Penguin: TTimer;
    Timer_Move_Arm_Seal:    TTimer;
    Timer_Vulture_Move:     TTimer;
    Timer_Wall_Move:        TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
  {}pm_12_displaycpu:   TMenuItem;
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                    (Sender: TObject);
    procedure FormActivate                  (Sender: TObject);
    procedure FormClose                     (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Show_All                  (b: boolean);

    procedure Vid_Show_All_Blocks           (b: boolean);
    procedure Vid_Show_All_Bombs            (b: boolean);
    procedure Vid_Show_All_Blocks_Pile      (b: boolean);
    procedure Vid_Show_All_Hero             (b: boolean);
    procedure Vid_Show_All_Vulture          (b: boolean);
    procedure Vid_Show_All_Wall             (b: boolean);

    procedure Vid_Show_Panel_Score          (p: integer; b: boolean);

    procedure Vid_Show_Block                (p: integer; b: boolean);
    procedure Vid_Show_Block_Pile           (p: integer; b: boolean);
    procedure Vid_Show_Bomb                 (p: integer; b: boolean);
    procedure Vid_Show_Char                 (p: integer; b: boolean);
    procedure Vid_Show_Vulture              (b: boolean);
    procedure Vid_Show_Wall                 (p: integer; b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                      (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove        (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_power_topMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_select_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_sound_topMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_penguin_atk_topMouseDown  (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_penguin_atk_topMouseUp    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_penguin_left_topMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_penguin_left_topMouseUp   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_penguin_right_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_penguin_right_topMouseUp  (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_seal_atk_topMouseDown     (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_seal_atk_topMouseUp       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_seal_left_topMouseDown    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_seal_left_topMouseUp      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_seal_right_topMouseDown   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_seal_right_topMouseUp     (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                   (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                     (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_power_down;
    procedure Inp_select_down;
    procedure Inp_sound_down;

    procedure Inp_peng_atk_down;            procedure Inp_peng_atk_up;
    procedure Inp_peng_left_down;           procedure Inp_peng_left_up;
    procedure Inp_peng_right_down;          procedure Inp_peng_right_up;
    procedure Inp_seal_atk_down;            procedure Inp_seal_atk_up;
    procedure Inp_seal_left_down;           procedure Inp_seal_left_up;
    procedure Inp_seal_right_down;          procedure Inp_seal_right_up;

{ * PROCEDURES: GAME                                                         * }
    procedure Gam_Check_Piled_Blocks        (p: integer);
    procedure Gam_Game_Init;
    procedure Gam_Game_Start;
    procedure Gam_Screen_Init;
    procedure Gam_Set_Mode;
    procedure Gam_Set_SelectMode2P;
    procedure Gam_Set_Sound;
    procedure Gam_Stop_All_Timers;
    procedure Gam_Throw_Bomb                (p: integer);

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_Block_GenerateTimer     (Sender: TObject);
    procedure Timer_Blocks_BonusTimer       (Sender: TObject);
    procedure Timer_CPU_AI_ExecuteTimer     (Sender: TObject);
    procedure Timer_GameTimer               (Sender: TObject);
    procedure Timer_Game_MissTimer          (Sender: TObject);
    procedure Timer_Game_OverTimer          (Sender: TObject);
    procedure Timer_Game_StartTimer         (Sender: TObject);
    procedure Timer_Move_Arm_PenguinTimer   (Sender: TObject);
    procedure Timer_Move_Arm_SealTimer      (Sender: TObject);
    procedure Timer_Vulture_MoveTimer       (Sender: TObject);
    procedure Timer_Wall_MoveTimer          (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
  {}procedure pm_12_displaycpuClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
  end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Penguin Land (Bandai, LSI Game Double Play)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.00';
  HH2_CONTENT_DATE    = '2017/12/04';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME       = HH2_CONTENT_NAME + ' Simulator';                             // application's name (appears on taskbar and Splash screen)
{}APP_VERSION    = 'Version ' + HH2_CONTENT_VERSION + ' * Final *';             // current version (appears on Splash screen)
{}APP_DATE       = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 4th December, 2017'; // release date (appears on Splash screen)

  URL_EMAIL      = 'mailto:' + HH2_CONTENT_EMAIL;                               // link to the email address
  URL_PAYPAL     = 'https://www.paypal.me/LucaAntignano';                       // link to the Paypal donation page
  URL_WEBSITE    = 'http://' + HH2_CONTENT_WEBSITE + '/';                       // link to the website

  REG_KEY        = 'Software\MADrigal\PenguinL50';                              // registry key

  HINT_CLICK     = 'click or press';
  HINT_POWER_OFF = 'to turn on the game';
  HINT_POWER_ON  = 'to turn off the game';
  HINT_SELECT_1P = 'to select ''2-player'' game';
  HINT_SELECT_2P = 'to select ''1-player'' game';
  HINT_SOUND_ON  = 'to turn sound effects off';
  HINT_SOUND_OFF = 'to turn sound effects on';

{}MENU_ITEMS = 13;
  NUM_KEYS = 9;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * display management * }
  bDisplayCPU:             boolean;                                             // manages the visualisation of the screen overlay in 1P mode

{ * sound management * }
  pfs_beep,
  pfs_block,
  pfs_bomb,
  pfs_bonus,
  pfs_gameover,
  pfs_got,
  pfs_start,
  pfs_throw:               PFSoundSample;                                       // sound samples

  bSound,                                                                       // sound enabled? (depending on selected driver on splash screen)
  bSoundGame:              boolean;                                             // sound enabled? (depending on setting of the "sound" switch)

{ * arrays of images * }
  aIm_panel_digit_pos:     array[1..4, 1..2] of TImage;                         // empty sprites to host score;
                                                                                  // 1st index: 1=peng left; 2=seal left; 3=peng right; 4=seal right
                                                                                  // 2nd index: tens and units

  aIm_panel_number:        array[0..9] of TImage;                               // digits for score

  aIm_hero:                array[1..2, 1..5] of record                          // penguin and seal (main sprites)
                                                  body: TImage;                   // 1st index: 1=peng; 2=seal
                                                  down: TImage;                   // 2nd index: X coordinate
                                                  up:  TImage;                    // record: body parts (body and hand up/down)
                                                end;

  aIm_block:               array[1..2, 0..4] of TImage;                         // moveable blocks (1st index=display (left/right); 2nd index=x coordinate, same as per heroes)
  aIm_block_pile:          array[1..2, 1..3] of TImage;                         // moveable blocks (1st index=display (left/right); 2nd index=y coordinate)
  aIm_bomb:                array[1..5, 0..11] of TImage;                        // bombs (indexes=x and y cordinates)
  aIm_vulture:             array[1..5, 5..6] of TImage;                         // vulture (indexes=x and y coordinates, same as bombs)
  aIm_wall:                array[1..2, 2..4] of TImage;                         // character on wall (1st index=display (left/right); 2nd index=x coordinate, same as per heroes)

{ * arrays of integers, booleans and records * }
  aBlockPileValue,                                                              // how many ice blocks piled to left or right display (1 or 2, respectively)
  aBombDirection,                                                               // Y value of destination of bomb (1=to penguin; 10=to seal)
  aHeroHasBomb,                                                                 // bomb held by the player (index=reference to the bomb)
  aPosBlockX,                                                                   // X coordinate of the ice block not being piled (0 to 4)
  aPosHeroX,                                                                    // X coordinate of characters (1=penguin, 2=seal)
  aScore:                  array[1..2] of integer;                              // score (index=reference to player being 1=penguin and 2=seal)
  aPosBomb:                array[1..2] of record                                // bombs (index=reference to bomb, col/row=coordinates)
                                            col,
                                            row: integer;
                                          end;

  bBombIsActive,                                                                // is bomb "in-game" (true); index=reference to the bomb
  bBombIsMoving,                                                                // is bomb with the vulture (true); index=reference to the bomb
  bBombJustGrabbed,                                                             // has bomb just been grabbed by the vulture (true); index=reference to the bomb
  bCharacterHasNoBlock:    array[1..2] of boolean;                              // does the character grab an ice block (true); index=character (1/2 = penguin/seal)

{ * arrays of mixed types *}
  aKeys:                   array[1..NUM_KEYS] of record                         // keys management (values)
                                                   int:  integer;
                                                   word: word;
                                                 end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=off; 1=select game; 2=game Penguin; 3=game Seal; 4=2P game; 5=game over
  iPosVultureX,                                                                 // X and Y coordinates of the vulture (x=5-6, y=1-5)
  iPosVultureY,                                                                 // X and Y coordinates of the vulture (x=5-6, y=1-5)
  iPosWallX,                                                                    // X coordinate of characters on the wall (2-4)
  iUseCPU,                                                                      // 0=2P game; 1=CPU is penguin; 2=CPU is seal
  iVultureHasBomb                                                               // does the vulture has a bomb? 0=no, 1/2 bomb no. 1/2 respectively
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible

  bControllerP1,                                                                // true if P1 (penguin) controller must be active
  bControllerP2,                                                                // true if P2 (seal) controller must be active
  bHumanP1,                                                                     // true if P1 (penguin) is human
  bHumanP2,                                                                     // true if P2 (seal) is human
  bKeyPressedP1,                                                                // true if any key controlling P1 is pressed
  bKeyPressedP2,                                                                // true if any key controlling P2 is pressed
  bSelectMode2P                                                                 // true if 2P mode selected
                         : boolean;

  // Game variables: loops
  iLoopBlocksBonus,
  iLoopGameActiveBomb,
  iLoopGameMiss,
  iLoopGameOver,
  iLoopGameStart
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel (score for seal and penguin) - LEFT DISPLAY
  aIm_panel_digit_pos[1, 1] := im_left_panel_score_peng_1; aIm_panel_digit_pos[1, 2] := im_left_panel_score_peng_2;
  aIm_panel_digit_pos[2, 1] := im_left_panel_score_seal_1; aIm_panel_digit_pos[2, 2] := im_left_panel_score_seal_2;

  // on-screen panel (score for seal and penguin) - RIGHT DISPLAY
  aIm_panel_digit_pos[3, 1] := im_right_panel_score_peng_1; aIm_panel_digit_pos[3, 2] := im_right_panel_score_peng_2;
  aIm_panel_digit_pos[4, 1] := im_right_panel_score_seal_1; aIm_panel_digit_pos[4, 2] := im_right_panel_score_seal_2;

  // on-screen panel (score digits) - LEFT AND RIGHT DISPLAYS
  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  // on-screen items - LEFT DISPLAY
  aIm_hero[1, 1].body := im_left_peng_1;
  aIm_hero[1, 2].body := im_left_peng_2_body; aIm_hero[1, 2].down := im_left_peng_2_down; aIm_hero[1, 2].up := im_left_peng_2_up;
  aIm_hero[1, 3].body := im_left_peng_3_body; aIm_hero[1, 3].down := im_left_peng_3_down; aIm_hero[1, 3].up := im_left_peng_3_up;
  aIm_hero[1, 4].body := im_left_peng_4_body; aIm_hero[1, 4].down := im_left_peng_4_down; aIm_hero[1, 4].up := im_left_peng_4_up;
  aIm_hero[1, 5].body := im_left_peng_5;

                                      aIm_bomb[2, 0] := im_left_explosion;   aIm_bomb[3, 0] := im_left_explosion;   aIm_bomb[4, 0] := im_left_explosion;
                                      aIm_bomb[2, 1] := im_left_bomb_2_1;    aIm_bomb[3, 1] := im_left_bomb_3_1;    aIm_bomb[4, 1] := im_left_bomb_4_1;
                                      aIm_bomb[2, 2] := im_left_bomb_2_2;    aIm_bomb[3, 2] := im_left_bomb_3_2;    aIm_bomb[4, 2] := im_left_bomb_4_2;
                                      aIm_bomb[2, 3] := im_left_bomb_2_3;    aIm_bomb[3, 3] := im_left_bomb_3_3;    aIm_bomb[4, 3] := im_left_bomb_4_3;
                                      aIm_bomb[2, 4] := im_left_bomb_2_4;    aIm_bomb[3, 4] := im_left_bomb_3_4;    aIm_bomb[4, 4] := im_left_bomb_4_4;
  aIm_bomb[1, 5] := im_left_bomb_1_5; aIm_bomb[2, 5] := im_left_bomb_2_5;    aIm_bomb[3, 5] := im_left_bomb_3_5;    aIm_bomb[4, 5] := im_left_bomb_4_5;

  aIm_wall[1, 2] := im_left_seal_wall_2;  aIm_wall[1, 3] := im_left_seal_wall_3;  aIm_wall[1, 4] := im_left_seal_wall_4;

  aIm_vulture[1, 5] := im_left_vulture_1_5; aIm_vulture[2, 5] := im_left_vulture_2_5;  aIm_vulture[3, 5] := im_left_vulture_3_5;  aIm_vulture[4, 5] := im_left_vulture_4_5;

  aIm_block[1, 0] := im_left_block_0;  aIm_block[1, 1] := im_left_block_1;  aIm_block[1, 2] := im_left_block_2;  aIm_block[1, 3] := im_left_block_3;  aIm_block[1, 4] := im_left_block_4;

  aIm_block_pile[1, 1] := im_left_block_5_down;  aIm_block_pile[1, 2] := im_left_block_5_middle;  aIm_block_pile[1, 3] := im_left_block_5_top;

  // on-screen items - RIGHT DISPLAY
  aIm_hero[2, 1].body := im_right_seal_1;
  aIm_hero[2, 2].body := im_right_seal_2_body; aIm_hero[2, 2].down := im_right_seal_2_down; aIm_hero[2, 2].up := im_right_seal_2_up;
  aIm_hero[2, 3].body := im_right_seal_3_body; aIm_hero[2, 3].down := im_right_seal_3_down; aIm_hero[2, 3].up := im_right_seal_3_up;
  aIm_hero[2, 4].body := im_right_seal_4_body; aIm_hero[2, 4].down := im_right_seal_4_down; aIm_hero[2, 4].up := im_right_seal_4_up;
  aIm_hero[2, 5].body := im_right_seal_5;

  aIm_bomb[2, 6] := im_right_bomb_2_6;   aIm_bomb[3, 6] := im_right_bomb_3_6;   aIm_bomb[4, 6] := im_right_bomb_4_6;   aIm_bomb[5, 6] := im_right_bomb_5_6;
  aIm_bomb[2, 7] := im_right_bomb_2_7;   aIm_bomb[3, 7] := im_right_bomb_3_7;   aIm_bomb[4, 7] := im_right_bomb_4_7;
  aIm_bomb[2, 8] := im_right_bomb_2_8;   aIm_bomb[3, 8] := im_right_bomb_3_8;   aIm_bomb[4, 8] := im_right_bomb_4_8;
  aIm_bomb[2, 9] := im_right_bomb_2_9;   aIm_bomb[3, 9] := im_right_bomb_3_9;   aIm_bomb[4, 9] := im_right_bomb_4_9;
  aIm_bomb[2, 10] := im_right_bomb_2_10; aIm_bomb[3, 10] := im_right_bomb_3_10; aIm_bomb[4, 10] := im_right_bomb_4_10;
  aIm_bomb[2, 11] := im_right_explosion; aIm_bomb[3, 11] := im_right_explosion; aIm_bomb[4, 11] := im_right_explosion;

  aIm_wall[2, 2] := im_right_peng_wall_2; aIm_wall[2, 3] := im_right_peng_wall_3; aIm_wall[2, 4] := im_right_peng_wall_4;

  aIm_vulture[2, 6] := im_right_vulture_2_6; aIm_vulture[3, 6] := im_right_vulture_3_6; aIm_vulture[4, 6] := im_right_vulture_4_6; aIm_vulture[5, 6] := im_right_vulture_5_6;

  aIm_block[2, 0] := im_right_block_0;  aIm_block[2, 1] := im_right_block_1; aIm_block[2, 2] := im_right_block_2; aIm_block[2, 3] := im_right_block_3; aIm_block[2, 4] := im_right_block_4;

  aIm_block_pile[2, 1] := im_right_block_5_down; aIm_block_pile[2, 2] := im_right_block_5_middle; aIm_block_pile[2, 3] := im_right_block_5_top;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Power"          (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Select"         (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Sound"          (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 43);                             // "Penguin Attack" (default: "SHIFT")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 36);                             // "Penguin Left"   (default: "Z")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 34);                             // "Penguin Right"  (default: "X")
  aKeys[7].int := iReg.ReadInteger('', 'Key7', 42);                             // "Seal Attack"    (default: "CTRL")
  aKeys[8].int := iReg.ReadInteger('', 'Key8', 39);                             // "Seal Left"      (default: "LEFT")
  aKeys[9].int := iReg.ReadInteger('', 'Key9', 40);                             // "Seal Right"     (default: "RIGHT")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to NUM_KEYS do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_power_top.Hint         := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_POWER_OFF; // "Power off"
  btn_select_top.Hint        := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_SELECT_1P; // "Select 1P"
  btn_sound_top.Hint         := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_SOUND_ON;  // "Sound on";
  btn_penguin_atk_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int];     // "Penguin Attack"
  btn_penguin_left_top.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];     // "Penguin Left"
  btn_penguin_right_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];     // "Penguin Right"
  btn_seal_atk_top.Hint      := HINT_CLICK + ' ' + aKeyHints[aKeys[7].int];     // "Seal Attack"
  btn_seal_left_top.Hint     := HINT_CLICK + ' ' + aKeyHints[aKeys[8].int];     // "Seal Left"
  btn_seal_right_top.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[9].int];     // "Seal Right"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_9_n2;
{}aMenuItem[9]  := pm_10_fullscreen;
{}aMenuItem[10] := pm_11_splashscreen;
{}aMenuItem[11] := pm_12_displaycpu;;
{}aMenuItem[12] := pm_13_n3;
{}aMenuItem[13] := pm_14_exit;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  iMode := 0;             Gam_Set_Mode;                                         // sets game to "off" mode
  bSelectMode2P := false; Gam_Set_SelectMode2P;                                 // sets "select" button to "1P"
  bSoundGame := true;     Gam_Set_Sound;                                        // sets audio to "on"

  bControllerP1 := false;                                                       // prevents hero movement
  bControllerP2 := false;                                                       // prevents hero movement

  Randomize;                                                                    // initializes the random numbers generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_beep);     FSOUND_Sample_Free(pfs_block);
  FSOUND_Sample_Free(pfs_bomb);     FSOUND_Sample_Free(pfs_bonus);
  FSOUND_Sample_Free(pfs_gameover); FSOUND_Sample_Free(pfs_got);
  FSOUND_Sample_Free(pfs_start);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides all sprites                                                   }
begin
  Vid_Show_All_Blocks(b);
  Vid_Show_All_Blocks_Pile(b);
  Vid_Show_All_Bombs(b);
  Vid_Show_All_Hero(b);
  Vid_Show_All_Vulture(b);
  Vid_Show_All_Wall(b);

  im_left_peng_6_climb.Visible := b;    im_right_seal_6_climb.Visible := b;
  im_left_peng_7_eat.Visible := b;      im_right_seal_7_eat.Visible := b;
  im_left_peng_7_eat_hand.Visible := b; im_right_seal_7_eat_hand.Visible := b;
  im_left_seal_eat.Visible := b;        im_right_peng_eat.Visible := b;
  im_left_seal_eat_hand.Visible := b;   im_right_peng_eat_hand.Visible := b;
  im_left_seal_blown.Visible := b;      im_right_seal_blown.Visible := b;
  im_left_peng_blown.Visible := b;      im_right_peng_blown.Visible := b;

  Vid_Show_Panel_Score(1, b);
  Vid_Show_Panel_Score(2, b);
end;





procedure TForm1.Vid_Show_All_Blocks(b: boolean);
{ displays/hides all moveeable ice blocks's sprites                            }
  var
    i, j: integer;
begin
  for i := 1 to 2 do
    for j := 0 to 4 do
      aIm_block[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Bombs(b: boolean);
{ displays/hides all bomb sprites                                              }
  var
    i, j: integer;
begin
  for i := 2 to 4 do
    for j := 0 to 11 do
      aIm_bomb[i, j].Visible := b;

  aIm_bomb[1, 5].Visible := b;
  aIm_bomb[5, 6].Visible := b;
end;





procedure TForm1.Vid_Show_All_Blocks_Pile(b: boolean);
{ displays/hides all piled ice blocks's sprites                                }
  var
    i, j: integer;
begin
  for i := 1 to 2 do
    for j := 1 to 3 do
      aIm_block_pile[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Hero(b: boolean);
{ displays/hides all heroes' sprites                                           }
  var
    i, j: integer;
begin
  for i := 1 to 2 do
    begin
      for j := 1 to 5 do
        aIm_hero[i, j].body.Visible := b;

      for j := 2 to 4 do
        begin
          aIm_hero[i, j].down.Visible := b;
          aIm_hero[i, j].up.Visible := b;
        end;
    end;
end;





procedure TForm1.Vid_Show_All_Vulture(b: boolean);
{ displays/hides all vulture's sprites                                         }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_vulture[i, 5].Visible := b;

  for i := 2 to 5 do
    aIm_vulture[i, 6].Visible := b;
end;





procedure TForm1.Vid_Show_All_Wall(b: boolean);
{ displays/hides all the character on the wall's sprites                       }
  var
    i, j: integer;
begin
  for i := 1 to 2 do
    for j := 2 to 4 do
      aIm_wall[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_Panel_Score(p: integer; b: boolean);
{ displays score in image format; 1st index is player (1=penguin, 2=seal)      }
{ the score is displayed simultaneously on the left and right displays         }
  var
    i,
    t, u: integer;
begin
  t := aScore[p] div 10;
  u := aScore[p] mod 10;

  if b then                                                                     // display or hide score/high-score?
    begin                                                                         // display
      if aScore[p] >  9 then
        begin
          aIm_panel_digit_pos[p, 1].Picture     := aIm_panel_number[t].Picture;     // left tens (digit)
          aIm_panel_digit_pos[p + 2, 1].Picture := aIm_panel_number[t].Picture;     // right tens (digit)
        end
      else
        begin
          aIm_panel_digit_pos[p, 1].Picture     := nil;                             // left tens (digit)
          aIm_panel_digit_pos[p + 2, 1].Picture := nil;                             // right tens (digit)
        end;

      aIm_panel_digit_pos[p, 2].Picture     := aIm_panel_number[u].Picture;         // left units (digit)
      aIm_panel_digit_pos[p + 2, 2].Picture := aIm_panel_number[u].Picture;         // right units (digit)
    end
  else                                                                            // hide
    for i := 1 to 2 do
      begin
        aIm_panel_digit_pos[p, i].Picture := nil;                                   // entire left display
        aIm_panel_digit_pos[p + 2, i].Picture := nil;                               // entire right display
      end;
end;





procedure TForm1.Vid_Show_Block(p: integer; b: boolean);
{ displays/hides the moveable block marked as "p" (1=penguin, 2=seal) in its   }
{ known pre-set position                                                       }
begin
  aIm_block[p, aPosBlockX[p]].Visible := b;
end;





procedure TForm1.Vid_Show_Block_Pile(p: integer; b: boolean);
{ displays/hides the piled block marked as "p" (1=penguin, 2=seal) in its      }
{ known pre-set position                                                       }
begin
  aIm_block_pile[p, aBlockPileValue[p]].Visible := b;
end;





procedure TForm1.Vid_Show_Bomb(p: integer; b: boolean);
{ displays/hides the bomb marked as "p" (p=1 or 2) in its known pre-set        }
{ position                                                                     }
begin
  aIm_bomb[aPosBomb[p].col, aPosBomb[p].row].Visible := b;
end;





procedure TForm1.Vid_Show_Char(p: integer; b: boolean);
{ displays/hides the hero marked as "p" (1=penguin, 2=seal) in its known       }
{ pre-set position, with hand always down (hand up not visible all the time    }
begin
  aIm_hero[p, aPosHeroX[p]].body.Visible := b;

  if (aPosHeroX[p] > 1)
    and (aPosHeroX[p] < 5) then
    begin
      aIm_hero[p, aPosHeroX[p]].down.Visible := b;
      aIm_hero[p, aPosHeroX[p]].up.Visible := false;
    end;
end;





procedure TForm1.Vid_Show_Vulture(b: boolean);
{ displays/hides the vulture at its pre-set coordinates                        }
begin
  aIm_vulture[iPosVultureX, iPosVultureY].Visible := b;
end;





procedure TForm1.Vid_Show_Wall(p: integer; b: boolean);
{ displays/hides the characters on the wall on its "p" position, on both the   }
{ left and right display simultaneously                                        }
begin
  aIm_wall[1, p].Visible := b;
  aIm_wall[2, p].Visible := b;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound and bSoundGame then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_power_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ power switch: CLICK                                                          }
begin
  if Button = mbLeft then
    Inp_power_down;
end;





procedure TForm1.btn_select_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ select switch: CLICK                                                         }
begin
  if Button = mbLeft then
    Inp_select_down;
end;





procedure TForm1.btn_sound_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ sound switch: CLICK                                                          }
begin
  if Button = mbLeft then
    Inp_sound_down;
end;





procedure TForm1.btn_penguin_atk_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ PENGUIN ATTACK button: PRESS                                                 }
begin
  if Button = mbLeft then
    Inp_peng_atk_down;
end;





procedure TForm1.btn_penguin_atk_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ PENGUIN ATTACK button: RELEASE                                               }
begin
  if Button = mbLeft then
    Inp_peng_atk_up;
end;





procedure TForm1.btn_penguin_left_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ PENGUIN LEFT button: PRESS                                                   }
begin
  if Button = mbLeft then
    Inp_peng_left_down;
end;





procedure TForm1.btn_penguin_left_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ PENGUIN LEFT button: RELEASE                                                 }
begin
  if Button = mbLeft then
    Inp_peng_left_up;
end;





procedure TForm1.btn_penguin_right_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ PENGUIN RIGHT button: PRESS                                                  }
begin
  if Button = mbLeft then
    Inp_peng_right_down;
end;





procedure TForm1.btn_penguin_right_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ PENGUIN RIGHT button: RELEASE                                                }
begin
  if Button = mbLeft then
    Inp_peng_right_up;
end;





procedure TForm1.btn_seal_atk_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SEAL ATTACK button: PRESS                                                    }
begin
  if Button = mbLeft then
    Inp_seal_atk_down;
end;





procedure TForm1.btn_seal_atk_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SEAL ATTACK button: RELEASE                                                  }
begin
  if Button = mbLeft then
    Inp_seal_atk_up;
end;





procedure TForm1.btn_seal_left_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SEAL LEFT button: PRESS                                                      }
begin
  if Button = mbLeft then
    Inp_seal_left_down;
end;





procedure TForm1.btn_seal_left_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SEAL LEFT button: RELEASE                                                    }
begin
  if Button = mbLeft then
    Inp_seal_left_up;
end;





procedure TForm1.btn_seal_right_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SEAL RIGHT button: PRESS                                                     }
begin
  if Button = mbLeft then
    Inp_seal_right_down;
end;





procedure TForm1.btn_seal_right_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SEAL RIGHT button: RELEASE                                                   }
begin
  if Button = mbLeft then
    Inp_seal_right_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_power_down   else                             // Power
  if Key = aKeys[2].word then Inp_select_down  else                             // Select
  if Key = aKeys[3].word then Inp_sound_down   else                             // Sound

  if Key = aKeys[4].word then Inp_peng_atk_down   else                          // Penguin attack
  if Key = aKeys[5].word then Inp_peng_left_down  else                          // Penguin left
  if Key = aKeys[6].word then Inp_peng_right_down else                          // Penguin right

  if Key = aKeys[7].word then Inp_seal_atk_down   else                          // Seal attack
  if Key = aKeys[8].word then Inp_seal_left_down  else                          // Seal left
  if Key = aKeys[9].word then Inp_seal_right_down;                              // Seal right
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[4].word then Inp_peng_atk_up   else                            // Penguin attack
  if Key = aKeys[5].word then Inp_peng_left_up  else                            // Penguin left
  if Key = aKeys[6].word then Inp_peng_right_up else                            // Penguin right

  if Key = aKeys[7].word then Inp_seal_atk_up   else                            // Seal attack
  if Key = aKeys[8].word then Inp_seal_left_up  else                            // Seal left
  if Key = aKeys[9].word then Inp_seal_right_up;                                // Seal right
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_power_down;
{ power switch: CLICK                                                          }
begin
  if iMode = 0 then                                                             // power off ->
    begin                                                                       //           -> game select
      iMode := 1;                                                                 
      btn_power_top.hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_POWER_ON;
    end
  else                                                                          // whichever ->
    begin                                                                       //           -> power off
      iMode := 0;
      btn_power_top.hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_POWER_OFF;
    end;

  Gam_Set_Mode;
end;





procedure TForm1.Inp_select_down;
{ select switch: CLICK                                                         }
begin
  bSelectMode2P := not bSelectMode2P;

  Gam_Set_SelectMode2P;
end;





procedure TForm1.Inp_sound_down;
{ sound switch: CLICK                                                          }
begin
  bSoundGame := not bSoundGame;

  Gam_Set_Sound;
end;





procedure TForm1.Inp_peng_atk_down;
{ PENGUIN ATTACK button: PRESS                                                 }
begin
  btn_penguin_atk_down.Visible := true;

  case iMode of
    1, 5: begin                                                                 // select game or game over ->
         iMode := 2 + Ord(bSelectMode2P) * 2;                                   //                          -> 2=game Penguin or 4=2P game
         Gam_Set_Mode;
       end;
    2, 4: if bControllerP1                                                      // in-game
         and (not bKeyPressedP1) then                         
         begin
           bKeyPressedP1 := true;

           if (aPosHeroX[1] = 1)                                                // penguin on the leftmost position
             and aIm_block[1, 0].Visible                                        // new ice block to the left
             and bCharacterHasNoBlock[1] then                                   // penguin has no ice block already
             begin
               aIm_block[2, 0].Visible := false;
               aIm_block[1, 0].Visible := false;

               aPosBlockX[1] := 1;
               Vid_Show_Block(1, true);
               bCharacterHasNoBlock[1] := false;

               Snd_Play(pfs_block);

               Timer_Block_Generate.Interval := 17000;
               Timer_Block_Generate.Enabled := true;
             end;

           if aHeroHasBomb[1] <> 0 then                                         // penguin holds a bomb
             begin                                                                // throws the bomb
               bControllerP1 := false;
               aIm_hero[1, aPosHeroX[1]].down.Visible := false;
               aIm_hero[1, aPosHeroX[1]].up.Visible := true;
               Gam_Throw_Bomb(aHeroHasBomb[1]);

               Snd_Play(pfs_throw);

               Timer_Move_Arm_Penguin.Interval := 300;
               Timer_Move_Arm_Penguin.Enabled := true;
             end;
         end;
  end; // case..of
end;





procedure TForm1.Inp_peng_atk_up;
{ PENGUIN ATTACK button: RELEASE                                               }
begin
  btn_penguin_atk_down.Visible := false;

  bKeyPressedP1 := false;
end;





procedure TForm1.Inp_peng_left_down;
{ PENGUIN LEFT button: PRESS                                                   }
  var
    i: integer;
begin
  btn_penguin_left_down.Visible := true;

  if bControllerP1
    and (not bKeyPressedP1)
    and (aPosHeroX[1] > 1) then
    begin
      bKeyPressedP1 := true;

      if (aHeroHasBomb[1] = 0)                                                  // penguin does not hold a bomb
        or ((aHeroHasBomb[1] <> 0) and (aPosHeroX[1] > 2)) then                 // or holds bomb/ice block and X is 3 or 4
        begin                                                                     // penguin moves
          Vid_Show_Char(1, false);
          Dec(aPosHeroX[1]);
          Vid_Show_Char(1, true);

          if aHeroHasBomb[1] <> 0 then                                            // bomb is also moved
            begin
              Vid_Show_Bomb(aHeroHasBomb[1], false);
              aPosBomb[aHeroHasBomb[1]].col := aPosHeroX[1];
              Vid_Show_Bomb(aHeroHasBomb[1], true);
            end
          else
            for i := 1 to 2 do
              if (aPosBomb[i].col = aPosHeroX[1])
                and (aPosBomb[i].row = 1) then
                begin
                  aHeroHasBomb[1] := i;
                  bBombIsMoving[i] := false;
                end;

          if not bCharacterHasNoBlock[1] then
            begin                                                                 // ice block is also moved
              Vid_Show_Block(1, false);
              aPosBlockX[1] := aPosHeroX[1];
              Vid_Show_Block(1, true);
            end;
        end;
    end;
end;





procedure TForm1.Inp_peng_left_up;
{ PENGUIN LEFT button: RELEASE                                                 }
begin
  btn_penguin_left_down.Visible:= false;

  bKeyPressedP1 := false;
end;





procedure TForm1.Inp_peng_right_down;
{ PENGUIN RIGHT button: PRESS                                                  }
  var
    i: integer;
begin
  btn_penguin_right_down.Visible := true;

  if bControllerP1
    and (not bKeyPressedP1)
    and (aPosHeroX[1] < 5) then
    begin
      bKeyPressedP1 := true;

      if (aHeroHasBomb[1] = 0)                                                  // penguin does not hold a bomb
        or ((aHeroHasBomb[1] <> 0) and (aPosHeroX[1] < 4)) then                 // or holds bomb/ice block and X is 2 or 3
        begin                                                                     // penguin moves
          Vid_Show_Char(1, false);
          Inc(aPosHeroX[1]);
          Vid_Show_Char(1, true);

          if aHeroHasBomb[1] <> 0 then                                            // bomb is also moved
            begin
              Vid_Show_Bomb(aHeroHasBomb[1], false);
              aPosBomb[aHeroHasBomb[1]].col := aPosHeroX[1];
              Vid_Show_Bomb(aHeroHasBomb[1], true);
            end
          else
            for i := 1 to 2 do
              if (aPosBomb[i].col = aPosHeroX[1])
                and (aPosBomb[i].row = 1) then
                begin
                  aHeroHasBomb[1] := i;
                  bBombIsMoving[i] := false;
                end;

          if not bCharacterHasNoBlock[1] then
            begin                                                                 // ice block is also moved
              Vid_Show_Block(1, false);

              if aPosHeroX[1] = 5 then                                              // to the pile
                begin
                  Inc(aBlockPileValue[1]);
                  Vid_Show_Block_Pile(1, true);

                  bCharacterHasNoBlock[1] := true;
                  Gam_Check_Piled_Blocks(1);

                  Snd_Play(pfs_block);
                end
              else                                                                  // or with the character
                begin
                  aPosBlockX[1] := aPosHeroX[1];
                  Vid_Show_Block(1, true);
                end;
            end;
        end;
  end;
end;





procedure TForm1.Inp_peng_right_up;
{ PENGUIN RIGHT button: RELEASE                                                }
begin
  btn_penguin_right_down.Visible := false;

  bKeyPressedP1 := false;
end;





procedure TForm1.Inp_seal_atk_down;
{ SEAL ATTACK button: PRESS                                                    }
begin
  btn_seal_atk_down.Visible := true;

  case iMode of
    1, 5: begin                                                                 // select game or game over ->
         iMode := 3 + Ord(bSelectMode2P) * 1;                                   //                          -> 3=game Seal or 4=2P game
         Gam_Set_Mode;
       end;
    3, 4: if bControllerP2                                                      // in-game
         and (not bKeyPressedP2) then
         begin
           bKeyPressedP2 := true;

           if (aPosHeroX[2] = 1)                                                // seal on the leftmost position
             and aIm_block[2, 0].Visible                                        // new ice block to the right
             and bCharacterHasNoBlock[2] then                                   // seal has no ice block already
             begin
               aIm_block[1, 0].Visible := false;
               aIm_block[2, 0].Visible := false;

               aPosBlockX[2] := 1;
               Vid_Show_Block(2, true);
               bCharacterHasNoBlock[2] := false;

               Snd_Play(pfs_block);

               Timer_Block_Generate.Interval := 17000;
               Timer_Block_Generate.Enabled := true;
             end;

           if aHeroHasBomb[2] <> 0 then                                         // penguin holds a bomb
             begin                                                                // throws the bomb
               bControllerP2 := false;
               aIm_hero[2, aPosHeroX[2]].down.Visible := false;
               aIm_hero[2, aPosHeroX[2]].up.Visible := true;
               Gam_Throw_Bomb(aHeroHasBomb[2]);

               Snd_Play(pfs_throw);
               
               Timer_Move_Arm_Seal.Interval := 300;
               Timer_Move_Arm_Seal.Enabled := true;
             end;
         end;
  end; // case..of
end;





procedure TForm1.Inp_seal_atk_up;
{ SEAL ATTACK button: RELEASE                                                  }
begin
  btn_seal_atk_down.Visible := false;

  bKeyPressedP2 := false;
end;





procedure TForm1.Inp_seal_left_down;
{ SEAL LEFT button: PRESS                                                      }
  var
    i: integer;
begin
  btn_seal_left_down.Visible := true;

  if bControllerP2
    and (not bKeyPressedP2)
    and (aPosHeroX[2] < 5) then
    begin
      bKeyPressedP2 := true;

      if (aHeroHasBomb[2] = 0)                                                  // seal does not hold a bomb
        or ((aHeroHasBomb[2] <> 0) and (aPosHeroX[2] < 4)) then                 // or holds bomb/ice block and X is 2 or 3
        begin                                                                     // seal moves
          Vid_Show_Char(2, false);
          Inc(aPosHeroX[2]);
          Vid_Show_Char(2, true);

          if aHeroHasBomb[2] <> 0 then                                            // bomb is also moved
            begin
              Vid_Show_Bomb(aHeroHasBomb[2], false);
              aPosBomb[aHeroHasBomb[2]].col := aPosHeroX[2];
              Vid_Show_Bomb(aHeroHasBomb[2], true);
            end
          else
            for i := 1 to 2 do
              if (aPosBomb[i].col = aPosHeroX[2])
                and (aPosBomb[i].row = 10) then
                begin
                  aHeroHasBomb[2] := i;
                  bBombIsMoving[i] := false;
                end;

          if not bCharacterHasNoBlock[2] then
            begin                                                                 // ice block is also moved
              Vid_Show_Block(2, false);

              if aPosHeroX[2] = 5 then                                              // to the pile
                begin
                  Inc(aBlockPileValue[2]);
                  Vid_Show_Block_Pile(2, true);

                  bCharacterHasNoBlock[2] := true;
                  Gam_Check_Piled_Blocks(2);

                  Snd_Play(pfs_block);
                end
              else
                begin                                                              // or with the character
                  aPosBlockX[2] := aPosHeroX[2];
                  Vid_Show_Block(2, true);
                end;
            end;
        end;
    end;
end;





procedure TForm1.Inp_seal_left_up;
{ SEAL LEFT button: RELEASE                                                    }
begin
  btn_seal_left_down.Visible := false;

  bKeyPressedP2 := false;
end;





procedure TForm1.Inp_seal_right_down;
{ SEAL RIGHT button: PRESS                                                     }
  var
    i: integer;
begin
  btn_seal_right_down.Visible := true;

  if bControllerP2
    and (not bKeyPressedP2)
    and (aPosHeroX[2] > 1) then
    begin
      bKeyPressedP2 := true;

      if (aHeroHasBomb[2] = 0)                                                  // seal does not hold a bomb
        or ((aHeroHasBomb[2] <> 0) and (aPosHeroX[2] > 2)) then                 // or holds bomb/ice block and X is 3 or 4
        begin                                                                     // seal moves
          Vid_Show_Char(2, false);
          Dec(aPosHeroX[2]);
          Vid_Show_Char(2, true);

          if aHeroHasBomb[2] <> 0 then                                            // bomb is also moved
            begin
              Vid_Show_Bomb(aHeroHasBomb[2], false);
              aPosBomb[aHeroHasBomb[2]].col := aPosHeroX[2];
              Vid_Show_Bomb(aHeroHasBomb[2], true);
            end
          else
            for i := 1 to 2 do
              if (aPosBomb[i].col = aPosHeroX[2])
                and (aPosBomb[i].row = 10) then
                begin
                  aHeroHasBomb[2] := i;
                  bBombIsMoving[i] := false;
                end;

          if not bCharacterHasNoBlock[2] then
            begin                                                                 // ice block is also moved
              Vid_Show_Block(2, false);
              aPosBlockX[2] := aPosHeroX[2];
              Vid_Show_Block(2, true);
            end;
        end;
    end;
end;





procedure TForm1.Inp_seal_right_up;
{ SEAL RIGHT button: RELEASE                                                   }
begin
  btn_seal_right_down.Visible := false;

  bKeyPressedP2 := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME                                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Check_Piled_Blocks(p: integer);
{ checks if P1 or P2 (as set through "p" parameter) has pile 3 ice blocks      }
begin
  if aBlockPileValue[p] = 3 then
    begin
      Gam_Stop_All_Timers;
      Snd_Silence;

      bControllerP1 := false;
      bControllerP2 := false;

      Snd_Play(pfs_bonus);

      iLoopBlocksBonus := 1;
      Timer_Blocks_Bonus.Interval := 1970;
      Timer_Blocks_Bonus.Enabled := true;
    end;
end;





procedure TForm1.Gam_Game_Init;
{ initializes the new game                                                     }
begin
  Gam_Screen_Init;

  bCharacterHasNoBlock[1] := true;
  bCharacterHasNoBlock[2] := true;

  aBlockPileValue[1] := 0;
  aBlockPileValue[2] := 0;

  aScore[1] := 15; Vid_Show_Panel_Score(1, true);
  aScore[2] := 15; Vid_Show_Panel_Score(2, true);

  iLoopGameStart := 1;
  Timer_Game_Start.Interval := 1;
  Timer_Game_Start.Enabled := true;
end;





procedure TForm1.Gam_Game_Start;
{ game action starts                                                           }
begin
  bControllerP1 := bHumanP1;
  bControllerP2 := bHumanP2;

  bKeyPressedP1 := not bHumanP1;
  bKeyPressedP2 := not bHumanP2;

  Timer_Vulture_Move.Interval := 700;
  Timer_Vulture_Move.Enabled := true;

  Timer_Wall_Move.Interval := 650;
  Timer_Wall_Move.Enabled := true;

  Timer_Block_Generate.Interval := 9400;
  Timer_Block_Generate.Enabled := true;

  if iUseCPU > 0 then
    Timer_CPU_AI_Execute.Enabled := true; // note timer interval was previously set in Gam_Set_Mode
end;





procedure TForm1.Gam_Screen_Init;
{ initializes the screen on a new game                                         }
begin
  // initializes characters
  aPosHeroX[1] := 4; Vid_Show_Char(1, true);
  aPosHeroX[2] := 2; Vid_Show_Char(2, true);
  iPosWallX := 3;    Vid_Show_Wall(iPosWallX, true);

  // sets vulture and bombs (position and direction)
  if Random(2) = 1 then                                                         // vulture on left display
    begin iPosVultureX := 1; iPosVultureY := 5; aBombDirection[1] := 0;  aPosBomb[2].col := 5; aPosBomb[2].row := 6; aBombDirection[2] := 11; end
  else                                                                          // vulture on right display
    begin iPosVultureX := 5; iPosVultureY := 6; aBombDirection[1] := 11; aPosBomb[2].col := 1; aPosBomb[2].row := 5; aBombDirection[2] := 0; end;

  Vid_Show_Vulture(true);

  // sets bombs
  aPosBomb[1].col := iPosVultureX;
  aPosBomb[1].row := iPosVultureY;
  Vid_Show_Bomb(1, true);

  bBombIsActive[1] := true;
  bBombIsActive[2] := false;

  bBombIsMoving[1] := false;
  bBombIsMoving[2] := false;

  aHeroHasBomb[1] := 0;
  aHeroHasBomb[2] := 0;

  iVultureHasBomb := 1;

  iLoopGameActiveBomb := 1;
end;





procedure TForm1.Gam_Set_Mode;
{ sets tabletop in off/game mode, depending on the value of the iMode variable }
  var
    i, j: integer;
begin
  case iMode of
    0: begin                                                                    // tabletop is off
         btn_power_down_on.Visible := false;

         im_left_background_disabled.Visible := false;
         im_right_background_disabled.Visible := false;

         Gam_Stop_All_Timers;
         Snd_Silence;

         Vid_Show_All(false);

         bControllerP1 := false;
         bControllerP2 := false;
       end;
    1: begin                                                                    // select mode
         btn_power_down_on.Visible := true;

         im_left_background_disabled.Visible := false;
         im_right_background_disabled.Visible := false;

         for i := 1 to 4 do                                                       // displays score panel on both screens
           for j := 1 to 2 do
             aIm_panel_digit_pos[i, j].Visible := true;

         aScore[1] := 15; Vid_Show_Panel_Score(1, true);                          // penguin
         aScore[2] := 15; Vid_Show_Panel_Score(2, true);                          // seal
       end;
    2: begin                                                                    // 1-player game (Penguin)
         im_left_background_disabled.Visible := false;
         im_right_background_disabled.Visible := not bDisplayCPU;

         for i := 1 to 1 do                                                       // displays score panel on left screen
           for j := 1 to 2 do
             aIm_panel_digit_pos[i, j].Visible := true;

         for i := 3 to 4 do                                                       // hides score panel on right screen
           for j := 1 to 2 do
             aIm_panel_digit_pos[i, j].Visible := false;

         bHumanP1 := true;
         bHumanP2 := false;
         iUseCPU := 2;
         Timer_CPU_AI_Execute.Interval := 450; // easy

         Vid_Show_All(false);
         Gam_Game_Init;

       end;
    3: begin                                                                    // 1-player game (Seal)
         im_left_background_disabled.Visible := not bDisplayCPU;
         im_right_background_disabled.Visible := false;

         for i := 1 to 2 do                                                       // hides score panel on left screen
           for j := 1 to 2 do
             aIm_panel_digit_pos[i, j].Visible := false;

         for i := 3 to 4 do                                                       // displays score panel on right screen
           for j := 1 to 2 do
             aIm_panel_digit_pos[i, j].Visible := true;

         bHumanP1 := false;
         bHumanP2 := true;
         iUseCPU := 1;
         Timer_CPU_AI_Execute.Interval := 400; // hard

         Vid_Show_All(false);
         Gam_Game_Init;
       end;
    4: begin                                                                    // 2-player game
         im_left_background_disabled.Visible := false;
         im_right_background_disabled.Visible := false;

         for i := 1 to 4 do                                                       // displays score panel on both screens
           for j := 1 to 2 do
             aIm_panel_digit_pos[i, j].Visible := true;
         
         bHumanP1 := true;
         bHumanP2 := true;
         iUseCPU := 0;

         Vid_Show_All(false);
         Gam_Game_Init;
       end;
    5: begin                                                                    // game over
         // does nothing
       end;
  end; // case..of
end;





procedure TForm1.Gam_Set_SelectMode2P;
{ sets the "Select" button                                                     }
begin
  btn_select_down_1P.Visible := not bSelectMode2P;

  if bSelectMode2P then
    btn_select_top.hint := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_SELECT_2P
  else
    btn_select_top.hint := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_SELECT_1P;
end;





procedure TForm1.Gam_Set_Sound;
{ sets the "Sound" button                                                      }
begin
  btn_sound_down_on.Visible := bSoundGame;

  if bSoundGame then                                                            // if sound enabled
    btn_sound_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_SOUND_ON // sets hint over the "sound" button
  else                                                                          // else
    begin
      Snd_Silence;                                                                // stops any currently playing sound
      btn_sound_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_SOUND_OFF; // sets hint over the "sound" button
    end;
end;





procedure TForm1.Gam_Stop_All_Timers;
{ stops all currently enabled timers                                           }
begin
  Timer_Block_Generate.Enabled   := false;
  Timer_Blocks_Bonus.Enabled     := false;
  Timer_CPU_AI_Execute.Enabled   := false;
  Timer_Game.Enabled             := false;
  Timer_Game_Miss.Enabled        := false;
  Timer_Game_Over.Enabled        := false;
  Timer_Game_Start.Enabled       := false;
  Timer_Move_Arm_Penguin.Enabled := false;
  Timer_Move_Arm_Seal.Enabled    := false;
  Timer_Vulture_Move.Enabled     := false;
  Timer_Wall_Move.Enabled        := false;
end;





procedure TForm1.Gam_Throw_Bomb(p: integer);
{ bomb ("p" is the identifier) is thrown back to the opponent                  }
begin
  case aPosBomb[p].row of
    1: begin
         Vid_Show_Bomb(p, false);
         aPosBomb[p].row := 2;

         if bBombIsActive[1] then Vid_Show_Bomb(1, true);
         if bBombIsActive[2] then Vid_Show_Bomb(2, true);

         aBombDirection[p] := 11;
         bBombIsMoving[p] := true;
       end;
   10: begin
         Vid_Show_Bomb(p, false);
         aPosBomb[p].row := 9;

         if bBombIsActive[1] then Vid_Show_Bomb(1, true);
         if bBombIsActive[2] then Vid_Show_Bomb(2, true);

         aBombDirection[p] := 0;
         bBombIsMoving[p] := true;
       end;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Block_GenerateTimer(Sender: TObject);
{ generates a new ice block at "0" position at regular intervals being         }
{ 9400ms at the beginning/reprise of the game and 17000ms in-game              }
begin
  aIm_block[1, 0].Visible := true;
  aIm_block[2, 0].Visible := true;

  Timer_Block_Generate.Enabled := false;
end;





procedure TForm1.Timer_Blocks_BonusTimer(Sender: TObject);
{ bonus intermission (penguin or seal have collected 3 ice blocks              }
  var
    i, j: integer;
begin
  case iLoopBlocksBonus of
    1: begin
         if aBlockPileValue[1] = 3 then                                         // penguin case
           begin aIm_hero[1, 5].body.Visible := false; im_left_peng_6_climb.Visible := true; im_right_peng_eat.Visible := true; end
         else                                                                   // seal case
           begin aIm_hero[2, 5].body.Visible := false; im_right_seal_6_climb.Visible := true; im_left_seal_eat.Visible := true; end;

         Timer_Blocks_Bonus.Interval := 705;
       end;
    2: begin
         if aBlockPileValue[1] = 3 then                                         // penguin case
           begin im_left_peng_6_climb.Visible := false; im_left_peng_7_eat.Visible := true; im_right_peng_eat.Visible := true; end
         else                                                                   // seal case
           begin im_right_seal_6_climb.Visible := false; im_right_seal_7_eat.Visible := true; im_left_seal_eat.Visible := true; end;

         Timer_Blocks_Bonus.Interval := 700;
       end;
    3, 5: begin
         if aBlockPileValue[1] = 3 then                                         // penguin case
           begin im_left_peng_7_eat_hand.Visible := true; im_right_peng_eat_hand.Visible := true; Dec(aScore[2]); Vid_Show_Panel_Score(2, true); end
         else                                                                   // seal case
           begin im_right_seal_7_eat_hand.Visible := true; im_left_seal_eat_hand.Visible := true; Dec(aScore[1]); Vid_Show_Panel_Score(1, true); end;

         Timer_Blocks_Bonus.Interval := 300;
       end;
    4, 6: begin
         if aBlockPileValue[1] = 3 then                                         // penguin case
           begin im_left_peng_7_eat_hand.Visible := false; im_right_peng_eat_hand.Visible := false; end
         else                                                                   // seal case
           begin im_right_seal_7_eat_hand.Visible := false; im_left_seal_eat_hand.Visible := false; end;

         if (aScore[1] = 0) or (aScore[2] = 0) then                             // score check
           begin                                                                  // game over
             iLoopGameOver := 1;
             Timer_Game_Over.Interval := 1;
             Timer_Game_Over.Enabled := true;

             Timer_Blocks_Bonus.Enabled := false;
           end
         else                                                                     // next step of the "ice blocks bonus" intermission
           Timer_Blocks_Bonus.Interval := 1415;
       end;
    7: begin
         if aBlockPileValue[1] = 3 then                                         // penguin case
           begin Vid_Show_Char(2, false); im_left_peng_7_eat.Visible := false; im_right_peng_eat.Visible := false; end
         else                                                                   // seal case
           begin Vid_Show_Char(1, false); im_right_seal_7_eat.Visible := false; im_left_seal_eat.Visible := false; end;

         Vid_Show_Wall(iPosWallX, false);
         Vid_Show_All_Blocks_Pile(false);
         Vid_Show_Vulture(false);

         aBlockPileValue[1] := 0;
         aBlockPileValue[2] := 0;

         // hides all bombs except the ones at the very base of the walls (rows 0 and 11)
         for i := 2 to 4 do
           for j := 1 to 10 do
             aIm_bomb[i, j].Visible := false;

         aIm_bomb[1, 5].Visible := false;
         aIm_bomb[5, 6].Visible := false;

         // game reprises
         Gam_Screen_Init;
         Gam_Game_Start;

         Timer_Blocks_Bonus.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopBlocksBonus);
end;





procedure TForm1.Timer_CPU_AI_ExecuteTimer(Sender: TObject);
{ the cpu-operated player performs action according to this routine, defining  }
{ its artificial intelligence                                                  }
  var
    i, j, iBomb, iCol: integer;
    tmpHasMoved: boolean;
begin
  tmpHasMoved := false;

  case iUseCPU of
    1: begin                                                                    // CPU is penguin
         if (aHeroHasBomb[1] > 0) then                                          // if penguin holds a bomb
           begin
             aIm_hero[1, aPosHeroX[1]].down.Visible := false;
             aIm_hero[1, aPosHeroX[1]].up.Visible := true;
             Gam_Throw_Bomb(aHeroHasBomb[1]);
             tmpHasMoved := true;

             Snd_Play(pfs_throw);
             
             Timer_Move_Arm_Penguin.Interval := 300;
             Timer_Move_Arm_Penguin.Enabled := true;
           end;

         if (aHeroHasBomb[1] = 0)                                               // if penguin does not hold a bomb
           and (not tmpHasMoved) then                                           // and has not moved yet
           begin
             iCol := 0;
             j := 1;

             while j < 5 do
               begin
                 for i := 2 to 4 do
                   begin
                     if aIm_bomb[i, j].Visible
                       and (iCol = 0) then
                       iCol := i;

                     if (iCol > 0)
                       and (aPosHeroX[1] < iCol) then
                       begin
                         Vid_Show_Char(1, false);
                         Inc(aPosHeroX[1]);
                         Vid_Show_Char(1, true);

                         if not bCharacterHasNoBlock[1] then
                           begin
                             Vid_Show_Block(1, false);
                             aPosBlockX[1] := aPosHeroX[1];
                             Vid_Show_Block(1, true);
                           end;

                         tmpHasMoved := true;
                         iCol := 0;
                         j := 5;

                         for ibomb := 1 to 2 do
                           if (aPosBomb[iBomb].col = aPosHeroX[1])
                             and (aPosBomb[iBomb].row = 1) then
                             begin
                               aHeroHasBomb[1] := iBomb;
                               bBombIsMoving[iBomb] := false;
                             end;
                       end;

                     if (iCol > 0)
                       and (aPosHeroX[1] > iCol) then
                       begin
                         Vid_Show_Char(1, false);
                         Dec(aPosHeroX[1]);
                         Vid_Show_Char(1, true);

                         if not bCharacterHasNoBlock[1] then
                           begin
                             Vid_Show_Block(1, false);
                             aPosBlockX[1] := aPosHeroX[1];
                             Vid_Show_Block(1, true);
                           end;

                         tmpHasMoved := true;
                         iCol := 0;
                         j := 5;

                         for iBomb := 1 to 2 do
                           if (aPosBomb[iBomb].col = aPosHeroX[1])
                             and (aPosBomb[iBomb].row = 1) then
                             begin
                               aHeroHasBomb[1] := iBomb;
                               bBombIsMoving[iBomb] := false;
                             end;
                       end;

                     if (iCol > 0)
                       and (aPosHeroX[1] = iCol) then
                       tmpHasMoved := true;
                   end;

                 Inc(j);
               end;
           end;

         if (not bCharacterHasNoBlock[1])                                       // if penguin does not hold an ice block
           and (not tmpHasMoved) then                                           // and has not moved yet
           begin
             Vid_Show_Char(1, false);
             Inc(aPosHeroX[1]);
             Vid_Show_Char(1, true);
             Vid_Show_Block(1, false);

             if aPosHeroX[1] = 5 then
               begin
                 Inc(aBlockPileValue[1]);
                 Vid_Show_Block_Pile(1, true);

                 bCharacterHasNoBlock[1] := true;
                 Gam_Check_Piled_Blocks(1);

                 Snd_Play(pfs_block);
               end
             else
               begin                                                              // ice block movement
                 aPosBlockX[1] := aPosHeroX[1];
                 Vid_Show_Block(1, true);
               end;

             tmpHasMoved := true;
           end;

         if aIm_block[1, 0].Visible                                             // if a new ice block has appeared
           and (not tmpHasMoved) then                                           // and penguin has not moved yet

           if (aPosHeroX[1] = 1) then
             begin
               aIm_block[2, 0].Visible := false;
               aIm_block[1, 0].Visible := false;

               aPosBlockX[1] := 1;
               Vid_Show_Block(1, true);
               bCharacterHasNoBlock[1] := false;

               Snd_Play(pfs_block);

               Timer_Block_Generate.Interval := 17000;
               Timer_Block_Generate.Enabled := true;
             end
           else
             begin
               Vid_Show_Char(1, false);
               Dec(aPosHeroX[1]);
               Vid_Show_Char(1, true);
             end;
       end; // case 1
    2: begin                                                                    // CPU is seal
         if (aHeroHasBomb[2] > 0) then                                          // if seal holds a bomb
           begin
             aIm_hero[2, aPosHeroX[2]].down.Visible := false;
             aIm_hero[2, aPosHeroX[2]].up.Visible := true;
             Gam_Throw_Bomb(aHeroHasBomb[2]);
             tmpHasMoved := true;

             Snd_Play(pfs_throw);
             
             Timer_Move_Arm_Seal.Interval := 300;
             Timer_Move_Arm_Seal.Enabled := true;
           end;

         if (aHeroHasBomb[2] = 0)                                               // if penguin does not hold a bomb
           and (not tmpHasMoved) then                                           // and has not moved yet
           begin
             iCol := 0;
             j := 10;

             while j > 6 do
               begin
                 for i := 2 to 4 do
                   begin
                     if aIm_bomb[i, j].Visible
                       and (iCol = 0) then
                       iCol := i;

                     if (iCol > 0)
                       and (aPosHeroX[2] < iCol) then
                       begin
                         Vid_Show_Char(2, false);
                         Inc(aPosHeroX[2]);
                         Vid_Show_Char(2, true);

                         if not bCharacterHasNoBlock[2] then
                           begin
                             Vid_Show_Block(2, false);
                             aPosBlockX[2] := aPosHeroX[2];
                             Vid_Show_Block(2, true);
                           end;

                         tmpHasMoved := true;
                         iCol := 0;
                         j := 6;

                         for iBomb := 1 to 2 do
                           if (aPosBomb[iBomb].col = aPosHeroX[2])
                             and (aPosBomb[iBomb].row = 10) then
                             begin
                               aHeroHasBomb[2] := iBomb;
                               bBombIsMoving[iBomb] := false;
                             end;
                       end;

                     if (iCol > 0)
                       and (aPosHeroX[2] > iCol) then
                       begin
                         Vid_Show_Char(2, false);
                         Dec(aPosHeroX[2]);
                         Vid_Show_Char(2, true);

                         if not bCharacterHasNoBlock[2] then
                           begin
                             Vid_Show_Block(2, false);
                             aPosBlockX[2] := aPosHeroX[2];
                             Vid_Show_Block(2, true);
                           end;

                         tmpHasMoved := true;
                         iCol := 0;
                         j := 6;

                         for iBomb := 1 to 2 do
                           if (aPosBomb[iBomb].col = aPosHeroX[2])
                             and (aPosBomb[iBomb].row = 10) then
                             begin
                               aHeroHasBomb[2] := iBomb;
                               bBombIsMoving[iBomb] := false;
                             end;
                       end;

                     if (iCol > 0)
                       and (aPosHeroX[2] = iCol) then
                       tmpHasMoved := true;
                   end;

                 Dec(j);
               end;
           end;

         if (not bCharacterHasNoBlock[2])                                       // if seal does not hold an ice block
           and (not tmpHasMoved) then                                           // and has not moved yet
           begin
             Vid_Show_Char(2, false);
             Inc(aPosHeroX[2]);
             Vid_Show_Char(2, true);
             Vid_Show_Block(2, false);

             if aPosHeroX[2] = 5 then
               begin
                 Inc(aBlockPileValue[2]);
                 Vid_Show_Block_Pile(2, true);

                 bCharacterHasNoBlock[2] := true;
                 Gam_Check_Piled_Blocks(2);

                 Snd_Play(pfs_block);
               end
             else
               begin                                                              // ice block movement
                 aPosBlockX[2] := aPosHeroX[2];
                 Vid_Show_Block(2, true);
               end;

             tmpHasMoved := true;
           end;

         if aIm_block[2, 0].Visible                                             // if a new ice block has appeared
           and (not tmpHasMoved) then                                           // and seal has not moved yet

           if (aPosHeroX[2] = 1) then
             begin
               aIm_block[1, 0].Visible := false;
               aIm_block[2, 0].Visible := false;

               aPosBlockX[2] := 1;
               Vid_Show_Block(2, true);
               bCharacterHasNoBlock[2] := false;

               Snd_Play(pfs_block);

               Timer_Block_Generate.Interval := 17000;
               Timer_Block_Generate.Enabled := true;
             end
           else
             begin
               Vid_Show_Char(2, false);
               Dec(aPosHeroX[2]);
               Vid_Show_Char(2, true);
             end;
       end; // case 2
  end; // case..of
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game action (mostly related to bombs movement and collisions            }
  var
    tmpMiss,               // collision check resulted in a miss (bomb explodes)
    tmpPlaySoundGot,       // plays either "got" or "beep" sound
    tmpHasMoved: boolean;  // has the active bomb moved already?
begin
  // loops active bomb
  if iLoopGameActiveBomb = 1 then
    iLoopGameActiveBomb := 2
  else
    iLoopGameActiveBomb := 1;

  if bBombIsMoving[iLoopGameActiveBomb] then
    begin
      tmpMiss := false;
      tmpPlaySoundGot := false;

      // COLLISION CHECK (1)

      // penguin holding a bomb and the other bomb is falling to him (at y=2) -> miss
      if (aPosBomb[iLoopGameActiveBomb].row = 2)
        and (aBombDirection[iLoopGameActiveBomb] = 0)
        and (aPosHeroX[1] = aPosBomb[iLoopGameActiveBomb].col)
        and (aHeroHasBomb[1] > 0) then
        begin
          Vid_Show_Bomb(iLoopGameActiveBomb, false);
          aPosBomb[iLoopGameActiveBomb].row := 0;
          Vid_Show_Bomb(iLoopGameActiveBomb, true);
          tmpMiss := true;
        end;

      // seal holding a bomb and the other bomb is falling to him (at y=9) -> miss
      if (aPosBomb[iLoopGameActiveBomb].row = 9)
        and (aBombDirection[iLoopGameActiveBomb] = 11)
        and (aPosHeroX[2] = aPosBomb[iLoopGameActiveBomb].col)
        and (aHeroHasBomb[2] > 0) then
        begin
          Vid_Show_Bomb(iLoopGameActiveBomb, false);
          aPosBomb[iLoopGameActiveBomb].row := 11;
          Vid_Show_Bomb(iLoopGameActiveBomb, true);
          tmpMiss := true;
        end;

      // MOVEMENT

      // case 1: bomb falling down to the penguin
      tmpHasMoved := false;

      if (not tmpMiss)
        and (aPosBomb[iLoopGameActiveBomb].row > aBombDirection[iLoopGameActiveBomb]) then
        begin
          tmpHasMoved := true;

          // move the bomb
          Vid_Show_Bomb(iLoopGameActiveBomb, false);
          Dec(aPosBomb[iLoopGameActiveBomb].row);
          Vid_Show_Bomb(1, true);

          if bBombIsActive[2] then
            Vid_Show_Bomb(2, true);

          // has the bomb hit a character on the wall? yes = throw it back down to the sender
          if (aPosBomb[iLoopGameActiveBomb].row = 7)
            and (aPosBomb[iLoopGameActiveBomb].col = iPosWallX) then
            aBombDirection[iLoopGameActiveBomb] := 11;

          // has the bomb hit the vulture? yes = bomb grabbed by the vulture
          if (aPosBomb[iLoopGameActiveBomb].row = 6)
            and (iVultureHasBomb = 0)
            and (iPosVultureY = 6)
            and (iPosVultureX = aPosBomb[iLoopGameActiveBomb].col) then
            begin                                     
              tmpPlaySoundGot := true;
              iVultureHasBomb := iLoopGameActiveBomb;
              bBombIsMoving[iLoopGameActiveBomb] := false;
              bBombJustGrabbed[iLoopGameActiveBomb] := true;
            end;
        end;

      // case 2: bomb falling down to the seal
      if (not tmpMiss)
        and (not tmpHasMoved)
        and (aPosBomb[iLoopGameActiveBomb].row < aBombDirection[iLoopGameActiveBomb]) then
        begin
          // move the bomb
          Vid_Show_Bomb(iLoopGameActiveBomb, false);
          Inc(aPosBomb[iLoopGameActiveBomb].row);
          Vid_Show_Bomb(1, true);

          if bBombIsActive[2] then
            Vid_Show_Bomb(2, true);

          // has the bomb hit a character on the wall? yes = throw it back down to the sender
          if (aPosBomb[iLoopGameActiveBomb].row = 4)
            and (aPosBomb[iLoopGameActiveBomb].col = iPosWallX) then
            aBombDirection[iLoopGameActiveBomb] := 0;

          // has the bomb hit the vulture? yes = bomb grabbed by the vulture
          if (aPosBomb[iLoopGameActiveBomb].row = 5)
            and (iVultureHasBomb = 0)
            and (iPosVultureY = 5)
            and (iPosVultureX = aPosBomb[iLoopGameActiveBomb].col) then
            begin
              tmpPlaySoundGot := true;
              iVultureHasBomb := iLoopGameActiveBomb;
              bBombIsMoving[iLoopGameActiveBomb] := false;
              bBombJustGrabbed[iLoopGameActiveBomb] := true;
            end;
        end;

      // COLLISION CHECK (2)

      // penguin NOT holding a bomb, a bomb is falling to him (at y=1) -> grab
      if (not tmpMiss)
        and (aPosBomb[iLoopGameActiveBomb].row = 1)
        and (aPosHeroX[1] = aPosBomb[iLoopGameActiveBomb].col)
        and (aHeroHasBomb[1] = 0) then
        begin
          aHeroHasBomb[1] := iLoopGameActiveBomb;
          bBombIsMoving[iLoopGameActiveBomb] := false;
          tmpPlaySoundGot := true;
        end;

      // seal NOT holding a bomb, a bomb is falling to him (at y=10) -> grab
      if (not tmpMiss)
        and (aPosBomb[iLoopGameActiveBomb].row = 10)
        and (aPosHeroX[2] = aPosBomb[iLoopGameActiveBomb].col)
        and (aHeroHasBomb[2] = 0) then
        begin
          aHeroHasBomb[2] := iLoopGameActiveBomb;
          bBombIsMoving[iLoopGameActiveBomb] := false;
          tmpPlaySoundGot := true;
        end;

      // checks whether a bomb made it to reach the ground
      if (aPosBomb[iLoopGameActiveBomb].row = aBombDirection[iLoopGameActiveBomb])
        and ((aPosBomb[iLoopGameActiveBomb].row = 0) or (aPosBomb[iLoopGameActiveBomb].row = 11)) then
        tmpMiss := true;

      // play sound (got or move)
      if tmpPlaySoundGot then
        Snd_Play(pfs_got)
      else
        Snd_Play(pfs_beep);

      // checks for exploded bomb and performs intermission accordingly
      if tmpMiss then
        begin
          Gam_Stop_All_Timers;

          bControllerP1 := false;
          bControllerP2 := false;

          Snd_Play(pfs_beep);

          iLoopGameMiss := 1;
          Timer_Game_Miss.Interval := 300;
          Timer_Game_Miss.Enabled := true;
        end;
    end;
end;





procedure TForm1.Timer_Game_MissTimer(Sender: TObject);
{ penguin or seal have exploded one bomb                                       }
  var
    i, j: integer;
begin
  case iLoopGameMiss of
    1: begin
         Snd_Play(pfs_bomb);

         Timer_Game_Miss.Interval := 600;
       end;
    2: begin
         if bBombIsActive[1] and (iLoopGameActiveBomb = 2) then Vid_Show_Bomb(1, false);
         if bBombIsActive[2] and (iLoopGameActiveBomb = 1) then Vid_Show_Bomb(2, false);

         if im_left_explosion.Visible then                                      // penguin miss
           begin
             Vid_Show_Char(1, false);

             im_left_peng_blown.Visible := true;
             im_right_peng_blown.Visible := true;

             if not bCharacterHasNoBlock[1] then
               begin
                 Vid_Show_Block(1, false);
                 bCharacterHasNoBlock[1] := true;
               end;

             Dec(aScore[1]);
           end
         else                                                                   // seal miss
           begin
             Vid_Show_Char(2, false);

             im_right_seal_blown.Visible := true;
             im_left_seal_blown.Visible := true;

             if not bCharacterHasNoBlock[2] then
               begin
                 Vid_Show_Block(2, false);
                 bCharacterHasNoBlock[2] := true;
               end;

             Dec(aScore[2]);
           end;
       end;  
    3: Timer_Game_Miss.Interval := 700;
    4: begin
         im_left_peng_blown.Visible := false;  im_right_peng_blown.Visible := false;

         im_right_seal_blown.Visible := false; im_left_seal_blown.Visible := false;

         aIm_block[1, 0].Visible := false;
         aIm_block[2, 0].Visible := false;

         Vid_Show_Vulture(false);
         Vid_Show_Wall(iPosWallX, false);
         Vid_Show_Char(1, false);
         Vid_Show_Char(2, false);
         Vid_Show_All_Bombs(false);

         Gam_Screen_Init;
         
         if not bCharacterHasNoBlock[1] then begin Vid_Show_Block(1, false); aPosBlockX[1] := aPosHeroX[1]; Vid_Show_Block(1, true); end;
         if not bCharacterHasNoBlock[2] then begin Vid_Show_Block(2, false); aPosBlockX[2] := aPosHeroX[2]; Vid_Show_Block(2, true); end;

         Vid_Show_Panel_Score(1, true);
         Vid_Show_Panel_Score(2, true);

         if (aScore[1] = 0)
           or (aScore[2] = 0) then
           begin
             iLoopGameOver := 1;
             Timer_Game_Over.Interval := 1;
             Timer_Game_Over.Enabled := true;
           end
         else
           Gam_Game_Start;

         Timer_Game_Miss.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopGameMiss);
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ plays "game over" tune then stops                                            }
begin
  case iLoopGameOver of
    1: Timer_Game_Over.Interval := 500;
    2: begin
         Snd_Silence;
         Snd_Play(pfs_gameover);

         Timer_Game_Over.Interval := 2000;
       end;
    3: begin
         iMode := 5; Gam_Set_Mode;

         Timer_Game_Over.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopGameOver);
end;





procedure TForm1.Timer_Game_StartTimer(Sender: TObject);
{ game start initialisation                                                    }
begin
  case iLoopGameStart of
    1: begin
         Snd_Play(pfs_start);

         Timer_Game_Start.Interval := 3400;
       end;
    2: begin
         Gam_Game_Start;

         Timer_Game_Start.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopGameStart);
end;





procedure TForm1.Timer_Move_Arm_PenguinTimer(Sender: TObject);
{ animation of the penguin's arm                                               }
  var
    i: integer;
begin
  aHeroHasBomb[1] := 0;
  bControllerP1 := bHumanP1;
  aIm_hero[1, aPosHeroX[1]].up.Visible := false;
  aIm_hero[1, aPosHeroX[1]].down.Visible := true;

  // if while returning a bomb, a new bomb is falling to the penguin -> grab it
  for i := 1 to 2 do
    if (aPosBomb[i].row = 1)
    and (aPosHeroX[1] = aPosBomb[i].col) then
    begin
      aHeroHasBomb[1] := i;
      bBombIsMoving[i] := false;
    end;

  Timer_Move_Arm_Penguin.Enabled := false;
end;





procedure TForm1.Timer_Move_Arm_SealTimer(Sender: TObject);
{ animation of the seal's arm                                                  }
  var
    i: integer;
begin
  aHeroHasBomb[2] := 0;
  bControllerP2 := bHumanP2;
  aIm_hero[2, aPosHeroX[2]].up.Visible := false;
  aIm_hero[2, aPosHeroX[2]].down.Visible := true;

  // if while returning a bomb, a new bomb is falling to the penguin -> grab it
  for i := 1 to 2 do
    if (aPosBomb[i].row = 10)
    and (aPosHeroX[2] = aPosBomb[i].col) then
    begin
      aHeroHasBomb[2] := i;
      bBombIsMoving[i] := false;
    end;

  Timer_Move_Arm_Seal.Enabled := false;
end;





procedure TForm1.Timer_Vulture_MoveTimer(Sender: TObject);
{ the vulture moves, grabs and drops bombs randomly                            }
begin
  // hides vulture at the current position (and the bomb, if with the vulture)
  Vid_Show_Vulture(false);

  if iVultureHasBomb <> 0 then
    Vid_Show_Bomb(iVultureHasBomb, false);

  // vulture movement (left-to-right)
  case iPosVultureY of
    5: begin                                                                    // left display (penguin)
         Inc(iPosVultureX);

         if iPosVultureX = 5 then
           begin
             iPosVultureY := 6;
             bBombJustGrabbed[1] := false;
             bBombJustGrabbed[2] := false;

             if not bBombIsActive[2] then
               begin
                 bBombIsActive[2] := true;
                 iVultureHasBomb := 2;
               end;
           end;
       end;
    6: begin                                                                    // right display (seal)
         Dec(iPosVultureX);

         if iPosVultureX = 1 then
           begin
             iPosVultureY := 5;
             bBombJustGrabbed[1] := false;
             bBombJustGrabbed[2] := false;

             if not bBombIsActive[2] then
               begin
                 bBombIsActive[2] := true;
                 iVultureHasBomb := 2;
               end;
           end;
       end;
  end; // case..of

  // displays the vulture in the new position
  Vid_Show_Vulture(true);

  // vulture grabs/drops bomb (randomly)
  if iVultureHasBomb <> 0 then
    begin
      if (iPosVultureX = 1)                                                     // if in (1,5)
        and (iPosVultureY = 5) then                
        begin                                                                     // sets bomb coordinates same as vulture's
          aPosBomb[iVultureHasBomb].col := 1;
          aPosBomb[iVultureHasBomb].row := 5;
        end;

      if (iPosVultureX > 1)                                                     // if in (1+,5)
        and (iPosVultureY = 5)                     
        and (not bBombJustGrabbed[iVultureHasBomb])
        and (Random(3) + 1 + iPosVultureX > 4) then
        begin
          Vid_Show_Bomb(iVultureHasBomb, false);
          aPosBomb[iVultureHasBomb].col := iPosVultureX;
          aPosBomb[iVultureHasBomb].row := 4;
          aBombDirection[iVultureHasBomb] := 0;
          bBombIsMoving[iVultureHasBomb] := true;
          Vid_Show_Bomb(iVultureHasBomb, true);
          iVultureHasBomb := 0;

          Snd_Play(pfs_beep);

          Timer_Game.Interval := 200;
          Timer_Game.Enabled := true;
        end;

      if (iPosVultureX = 5)                                                     // if in (5,6)
        and (iPosVultureY = 6) then                
        begin                                                                     // sets bomb coordinates same as vulture's
          aPosBomb[iVultureHasBomb].col := 5;
          aPosBomb[iVultureHasBomb].row := 6;
        end;

      if (iPosVultureX < 5)                                                     // if in (4-,6)
        and (iPosVultureY = 6)                     
        and (not bBombJustGrabbed[iVultureHasBomb])
        and (Random(3) - 4 + iPosVultureX < 1) then
        begin
          Vid_Show_Bomb(iVultureHasBomb, false);
          aPosBomb[iVultureHasBomb].col := iPosVultureX;
          aPosBomb[iVultureHasBomb].row := 7;
          aBombDirection[iVultureHasBomb] := 11;
          bBombIsMoving[iVultureHasBomb] := true;
          Vid_Show_Bomb(iVultureHasBomb, true);
          iVultureHasBomb := 0;

          Snd_Play(pfs_beep);

          Timer_Game.Interval := 200;
          Timer_Game.Enabled := true;
        end;
    end;

  // updates the coordinates of the bomb with the vulture) and displays it
  if iVultureHasBomb <> 0 then
    begin
      aPosBomb[iVultureHasBomb].col := iPosVultureX;
      aPosBomb[iVultureHasBomb].row := iPosVultureY;
      Vid_Show_Bomb(iVultureHasBomb, true);
    end;
end;





procedure TForm1.Timer_Wall_MoveTimer(Sender: TObject);
{ the characters on the wall move randomly                                     }
begin
  case iPosWallX of
    2: if Random(2) = 0 then
         begin
           Vid_Show_Wall(iPosWallX, false);
           Inc(iPosWallX);
           Vid_Show_Wall(iPosWallX, true);
         end;
    3: case Random(3) of
         0: begin
              Vid_Show_Wall(iPosWallX, false);
              Inc(iPosWallX);
              Vid_Show_Wall(iPosWallX, true);
             end;
         1: begin
              Vid_Show_Wall(iPosWallX, false);
              Dec(iPosWallX);
              Vid_Show_Wall(iPosWallX, true);
            end;
       end; // case.. of
    4: if Random(2) = 0 then
         begin
           Vid_Show_Wall(iPosWallX, false);
           Dec(iPosWallX);
           Vid_Show_Wall(iPosWallX, true);
         end;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);
{}pm_12_displaycpu.Checked := bDisplayCPU;

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_12_displaycpuClick(Sender: TObject);
{ displays the "tranlucent" controller overlay on the directional cross        }
begin
{}pm_12_displaycpu.Checked := not pm_12_displaycpu.Checked;
{}bDisplayCPU := pm_12_displaycpu.Checked;

{}iReg.WriteBool('', 'DisplayCPU', pm_12_displaycpu.Checked);

  // displays/hides the overlay if "in-game"
  case iMode of
    2: begin
         im_left_background_disabled.Visible := false;
         im_right_background_disabled.Visible := not bDisplayCPU;
       end;
    3: begin
         im_left_background_disabled.Visible := not bDisplayCPU;
         im_right_background_disabled.Visible := false;
       end;
  end; // case..of
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Size := -12;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.

